@@ -4,13 +4,12 @@ use strict;
 use warnings;
 
 use Net::Async::MPD;
-use PerlX::Maybe;
 
 # use Log::Any::Adapter;
 # Log::Any::Adapter->set( 'Stderr', log_level => 'trace' );
 
 my $mpd = Net::Async::MPD->new(
-  maybe host => $ARGV[0],
+  $ARGV[0] ? ( host => $ARGV[0] ) : (),
   auto_connect => 1,
 );
 
